package com.yakindu.ea.examples.orderingsoftware.template.core

import java.util.Set
import org.eclipse.uml2.uml.Class
import org.eclipse.uml2.uml.DataType
import org.eclipse.uml2.uml.Property
import org.eclipse.xtend2.lib.StringConcatenation

import static extension com.yakindu.ea.examples.orderingsoftware.template.util.JavaUtils.*

abstract class ClassTemplate extends AbstractTemplate {
	def String generate(Class element) '''
		«printPackageDeclaration(element)»
		
		«printImports(element)»
		
		«printClassComment»
		«printClassDeclaration(element)» {
		
			«printAttributes(element)»
		
			«printGettersAndSetters(element)»
		}
	'''
	
	protected def String printClassComment() '''
		/**
		 * This file was generated by {@link «class.name»}!
		 */
	'''
	
	protected def String printPackageDeclaration(Class element) '''
		package «element.package.javaQualifiedName»;
	'''
	
	protected def String printClassDeclaration(Class element) '''
		«val superType = element.generals.findFirst[true]?.name»
		public«IF element.isAbstract» abstract«ENDIF» class «element.name»«IF !superType.isNullOrEmpty» extends «superType»«ENDIF»'''
	
	protected def String printGettersAndSetters(Class element) '''
		«FOR attribute : element.ownedAttributes SEPARATOR StringConcatenation.DEFAULT_LINE_DELIMITER»
			«printGetter(attribute)»
			«IF 1 == attribute.upper»
				
				«printSetter(attribute)»
			«ENDIF»
		«ENDFOR»
	'''
	
	protected def String printGetter(Property attribute) '''
		«IF 1 != attribute.upper»
			public List<«attribute.type?.name»> get«attribute.name.toFirstUpper»() {
				return «attribute.name»;
			}
		«ELSE»
			public «attribute.type?.name» get«attribute.name.toFirstUpper»() {
				return «attribute.name»;
			}
		«ENDIF»
	'''

	protected def String printSetter(Property attribute) '''
		public void set«attribute.name.toFirstUpper»(«attribute.type?.name» «attribute.name») {
			this.«attribute.name» = «attribute.name»;
		}
	'''
	
	protected def String printAttributes(Class element) '''
		«FOR attribute : element.ownedAttributes SEPARATOR StringConcatenation.DEFAULT_LINE_DELIMITER»
			«printAttribute(attribute)»
		«ENDFOR»
	'''
	
	protected def String printAttribute(Property attribute) '''
		«IF 1 != attribute.upper»
			private «IF makeFinal(attribute)»final «ENDIF» List<«attribute.type?.name»> «attribute.name» = new LinkedList<«attribute.type?.name»>(); 
		«ELSE»
			private «attribute.type.name» «attribute.name»; 
		«ENDIF»
	'''
	
	protected def boolean makeFinal(Property attribute) {
		return true
	}
	
	protected def String printImports(Class element) {
		val datatypes = collectImports(element)
		return '''
			«FOR datatype : datatypes.sort»
				import «datatype»;
			«ENDFOR»
		'''
	}
	
	protected def Set<String> collectImports(Class element) {
		val datatypes = newHashSet()
		element.ownedAttributes.forEach[attribute |
			collectImports(attribute, datatypes)
		]
		return datatypes
	}
	
	protected def void collectImports(Property attribute, Set<String> datatypes) {
		if (attribute.type instanceof DataType) {
			if ("java.lang" != attribute.type.package.javaQualifiedName) {
				datatypes.add((attribute.type as DataType).javaQualifiedName)
			}
		}
		if (1 != attribute.upper) {
			datatypes.add("java.util.List")
			datatypes.add("java.util.LinkedList")
		}
	}
}