package com.yakindu.ea.examples.orderingsoftware.template.hbase

import com.yakindu.ea.examples.orderingsoftware.template.core.PersistenceManagerTemplate
import com.yakindu.bridges.ea.examples.runtime.codegen.EACodegen
import com.yakindu.bridges.ea.examples.runtime.codegen.EACodegenFile
import java.math.BigInteger
import java.util.Date
import org.eclipse.core.resources.IFile
import org.eclipse.uml2.uml.DataType
import org.eclipse.uml2.uml.Package
import org.eclipse.xtend2.lib.StringConcatenation

import static extension com.yakindu.ea.examples.orderingsoftware.template.util.JavaUtils.*

class HBasePersistenceManagerTemplate extends PersistenceManagerTemplate {
	@EACodegenFile
	override path(Package element, IFile defaultFile) {
		return getTargetFilePath("hbase", "com/example/orderingsoftware/" + super.path(element, defaultFile).name)
	}
	
	@EACodegen("java")
	override generate(Package element) {
		super.generate(element)
	}
	
	override protected generatePersistenceManager(Package element) '''
		«printPackageDeclaration(element)»

		import java.io.IOException;
		import java.math.BigInteger;
		import java.text.SimpleDateFormat;
		import java.util.UUID;

		import org.apache.hadoop.conf.Configuration;
		import org.apache.hadoop.fs.Path;
		import org.apache.hadoop.hbase.HBaseConfiguration;
		import org.apache.hadoop.hbase.HColumnDescriptor;
		import org.apache.hadoop.hbase.HTableDescriptor;
		import org.apache.hadoop.hbase.TableName;
		import org.apache.hadoop.hbase.client.Admin;
		import org.apache.hadoop.hbase.client.Connection;
		import org.apache.hadoop.hbase.client.ConnectionFactory;
		import org.apache.hadoop.hbase.client.Get;
		import org.apache.hadoop.hbase.client.HBaseAdmin;
		import org.apache.hadoop.hbase.client.Put;
		import org.apache.hadoop.hbase.client.Result;
		import org.apache.hadoop.hbase.client.Table;
		import org.apache.hadoop.hbase.util.Bytes;

		«printClassComment»
		public class PersistenceManager implements AutoCloseable {
			public static final String ID_SEPARATOR = ",";

			«FOR type : collectClasses(element) SEPARATOR StringConcatenation.DEFAULT_LINE_DELIMITER»
				public static final TableName «type.name.toUpperCase»_TABLE_NAME = TableName.valueOf("«type.name»Table");

				«FOR attribute : type.ownedAttributes SEPARATOR StringConcatenation.DEFAULT_LINE_DELIMITER»
					public static final byte[] «type.name.toUpperCase»_«attribute.name.toUpperCase»_QUALIFIER = "«attribute.name»".getBytes();
				«ENDFOR»
			«ENDFOR»

			public static final byte[] MODEL_FAMILY = "Model".getBytes();

			private final Connection connection;

			«FOR type : collectClasses(element) SEPARATOR StringConcatenation.DEFAULT_LINE_DELIMITER»
				private final Table «type.name.toFirstLower»Table;
			«ENDFOR»

			private final SimpleDateFormat dateFormat = new SimpleDateFormat();

			public PersistenceManager() {
				try {
					Configuration config = HBaseConfiguration.create();
					String path = getClass().getClassLoader().getResource("hbase-site.xml").getPath();
					config.addResource(new Path(path));
					HBaseAdmin.checkHBaseAvailable(config);

					connection = ConnectionFactory.createConnection(config);

					Admin admin = connection.getAdmin();
					«FOR type : collectClasses(element)»
						«type.name.toFirstLower»Table = ensureTableExists(admin, «type.name.toUpperCase»_TABLE_NAME, MODEL_FAMILY);
					«ENDFOR»
				} catch (Exception e) {
					throw new RuntimeException(e.getMessage(), e);
				}
			}

			protected Table ensureTableExists(Admin admin, TableName tableName, byte[] family) throws IOException {
				if (!admin.tableExists(tableName)) {
					HTableDescriptor desc = new HTableDescriptor(tableName);
					desc.addFamily(new HColumnDescriptor(family));
					admin.createTable(desc);
				}
				return connection.getTable(ARTICLE_TABLE_NAME);
			}

			public boolean save(AbstractIDObject entity) {
				try {
					«FOR type : collectClasses(element)»
					if (entity instanceof «type.name») {
						save((«type.name») entity);
						return true;
					}
					«ENDFOR»
					return false;
				} catch (IOException e) {
					throw new RuntimeException(e.getMessage(), e);
				}
			}

			«FOR type : collectClasses(element) SEPARATOR StringConcatenation.DEFAULT_LINE_DELIMITER»
				protected void save(«type.name» entity) throws IOException {
					if (entity.getId() == null) {
						entity.setId(UUID.randomUUID());
					}
					Put put = new Put(entity.getId().toString().getBytes());
					«FOR attribute : type.attributes»
						«IF attribute.type instanceof DataType»
							«IF typeof(Date).canonicalName == attribute.type.javaQualifiedName»
								put.addImmutable(MODEL_FAMILY, «type.name.toUpperCase»_«attribute.name.toUpperCase»_QUALIFIER, dateFormat.format(entity.get«attribute.name.toFirstUpper»()).toString().getBytes());
							«ELSE»
								«IF typeof(BigInteger).canonicalName == attribute.type.javaQualifiedName»
									put.addImmutable(MODEL_FAMILY, «type.name.toUpperCase»_«attribute.name.toUpperCase»_QUALIFIER, entity.get«attribute.name.toFirstUpper»() != null ? entity.get«attribute.name.toFirstUpper»().toString().getBytes() : null);
								«ELSE»
									put.addImmutable(MODEL_FAMILY, «type.name.toUpperCase»_«attribute.name.toUpperCase»_QUALIFIER, entity.get«attribute.name.toFirstUpper»() != null ? entity.get«attribute.name.toFirstUpper»().getBytes() : null);
								«ENDIF»
							«ENDIF»
						«ELSE»
							«IF 1 == attribute.upper»
								if (entity.get«attribute.name.toFirstUpper»() != null) {
									if (entity.get«attribute.name.toFirstUpper»().getId() == null) {
										save(entity.get«attribute.name.toFirstUpper»());
									}
									put.addImmutable(MODEL_FAMILY, «type.name.toUpperCase»_«attribute.name.toUpperCase»_QUALIFIER, entity.get«attribute.name.toFirstUpper»().getId().toString().getBytes());
								}
							«ELSE»
								StringBuilder itemIds = new StringBuilder();
								boolean afterFirst = false;
								for (OrderItem orderItem : entity.get«attribute.name.toFirstUpper»()) {
									if (afterFirst) {
										itemIds.append(ID_SEPARATOR);
									} else {
										afterFirst = true;
									}
									if (orderItem.getId() == null) {
										save(orderItem);
									}
									itemIds.append(orderItem.getId());
								}
								put.addImmutable(MODEL_FAMILY, «type.name.toUpperCase»_«attribute.name.toUpperCase»_QUALIFIER, itemIds.toString().getBytes());
							«ENDIF»
						«ENDIF»
					«ENDFOR»
					«type.name.toFirstLower»Table.put(put);
				}
			«ENDFOR»

			«FOR type : collectClasses(element) SEPARATOR StringConcatenation.DEFAULT_LINE_DELIMITER»
				public «type.name» get«type.name.toFirstUpper»(UUID id) {
					try {
						if (id != null) {
							Get get = new Get(id.toString().getBytes());
							if («type.name.toFirstLower»Table.exists(get)) {
								Result r = «type.name.toFirstLower»Table.get(get);
								«type.name» result = new «type.name»();
								result.setId(id);
								«FOR attribute : type.attributes»
									«IF attribute.type instanceof DataType»
										«IF typeof(Date).canonicalName == attribute.type.javaQualifiedName»
											result.set«attribute.name.toFirstUpper»(dateFormat.parse(Bytes.toString(r.getValue(MODEL_FAMILY, «type.name.toUpperCase»_«attribute.name.toUpperCase»_QUALIFIER))));
										«ELSE»
											«IF typeof(BigInteger).canonicalName == attribute.type.javaQualifiedName»
												String «attribute.name.toFirstLower» = Bytes.toString(r.getValue(MODEL_FAMILY, «type.name.toUpperCase»_«attribute.name.toUpperCase»_QUALIFIER));
												if («attribute.name.toFirstLower» != null && !«attribute.name.toFirstLower».isEmpty()) {
													result.set«attribute.name.toFirstUpper»(new BigInteger(«attribute.name.toFirstLower»));
												}
											«ELSE»
												result.set«attribute.name.toFirstUpper»(Bytes.toString(r.getValue(MODEL_FAMILY, «type.name.toUpperCase»_«attribute.name.toUpperCase»_QUALIFIER)));
											«ENDIF»
										«ENDIF»
									«ELSE»
										«IF 1 == attribute.upper»
											String «attribute.name.toFirstLower»Id = Bytes.toString(r.getValue(MODEL_FAMILY, «type.name.toUpperCase»_«attribute.name.toUpperCase»_QUALIFIER));
											if («attribute.name.toFirstLower»Id != null && !«attribute.name.toFirstLower»Id.isEmpty()) {
												result.set«attribute.name.toFirstUpper»(get«attribute.type.name.toFirstUpper»(UUID.fromString(«attribute.name.toFirstLower»Id)));			    	
											}
										«ELSE»
											String «attribute.name.toFirstLower»Ids = Bytes.toString(r.getValue(MODEL_FAMILY, «type.name.toUpperCase»_«attribute.name.toUpperCase»_QUALIFIER));
											if («attribute.name.toFirstLower»Ids != null && !«attribute.name.toFirstLower»Ids.isEmpty()) {
												String[] ids = «attribute.name.toFirstLower»Ids.split(ID_SEPARATOR);
												for (String currentId : ids) {
													«attribute.type.name» item = get«attribute.type.name.toFirstUpper»(UUID.fromString(currentId));
													if (item != null) {
														result.get«attribute.name.toFirstUpper»().add(item);
													}
												}
											}
										«ENDIF»
									«ENDIF»
								«ENDFOR»
								return result;
							}
						}
						return null;
					} catch (Exception e) {
						throw new RuntimeException(e.getMessage(), e);
					}
				}
			«ENDFOR»

			@Override
			public void close() {
				try {
					connection.close();
				} catch (IOException e) {
					throw new RuntimeException(e.getMessage(), e);
				}
			}
		}
	'''
}